@@ -11,18 +11,18 @@
 #include <custom_conf.h>
 #else
 #define PROTOCOL_VERSION		10
-#define MYSQL_SERVER_VERSION		"11.4.4-MariaDB"
+#define MYSQL_SERVER_VERSION		"11.4.5-MariaDB"
 #define MYSQL_BASE_VERSION		"mysqld-11.4"
 #define MARIADB_BASE_VERSION		"mariadb-11.4"
 #define MARIADBD_BASE_VERSION		"mariadbd-11.4"
 #define MYSQL_SERVER_SUFFIX_DEF		""
 #define FRM_VER				6
-#define MYSQL_VERSION_ID		110404
+#define MYSQL_VERSION_ID		110405
 #define MARIADB_PORT                    3306
 #define MYSQL_PORT_DEFAULT		0
 #define MARIADB_UNIX_ADDR               "/var/run/mysql/mysql.sock"
 #define MYSQL_CONFIG_NAME		"my"
-#define MYSQL_COMPILATION_COMMENT	"OpenBSD port: mariadb-server-11.4.4v1"
+#define MYSQL_COMPILATION_COMMENT	"OpenBSD port: mariadb-server-11.4.5p0v1"
 #define SERVER_MATURITY_LEVEL           MariaDB_PLUGIN_MATURITY_STABLE
 
 #define MYSQL_PORT                      MARIADB_PORT
