@@ -136,6 +136,21 @@ public:
       return Fbt_null(item, false).is_null();
     }
 
+    /*
+      Check at fix_fields() time if any of the items can return a nullable
+      value on conversion to Fbt.
+    */
+    static bool fix_fields_maybe_null_on_conversion_to_fbt(Item **items,
+                                                           uint count)
+    {
+      for (uint i= 0; i < count; i++)
+      {
+        if (Fbt::fix_fields_maybe_null_on_conversion_to_fbt(items[i]))
+          return true;
+      }
+      return false;
+    }
+
   public:
 
     Fbt(Item *item, bool *error, bool warn= true)
@@ -164,6 +179,10 @@ public:
                                              FbtImpl::max_char_length()+1));
       return false;
     }
+    bool to_bool() const
+    {
+      return !this->only_zero_bytes(m_buffer, FbtImpl::binary_length());
+    }
     int cmp(const Binary_string &other) const
     {
       return FbtImpl::cmp(FbtImpl::to_lex_cstring(), other.to_lex_cstring());
@@ -238,6 +257,10 @@ public:
     {
       return singleton();
     }
+    bool val_bool() override
+    {
+      return m_value.to_bool();
+    }
     longlong val_int() override
     {
       return 0;
@@ -275,8 +298,9 @@ public:
       str->append(tmp);
       str->append('\'');
     }
-    Item *get_copy(THD *thd) override
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_literal_fbt>(thd, this); }
+    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
 
     // Non-overriding methods
     void set_value(const Fbt &value)
@@ -567,7 +591,7 @@ public:
       return do_field_string;
     }
 
-    static void do_field_fbt_native_to_binary(Copy_field *copy)
+    static void do_field_fbt_native_to_binary(const Copy_field *copy)
     {
       NativeBuffer<FbtImpl::binary_length()+1> res;
       copy->from_field->val_native(&res);
@@ -622,8 +646,9 @@ public:
         return NULL;
       return new (thd->mem_root) Item_literal_fbt(thd, tmp);
     }
-    bool can_optimize_keypart_ref(const Item_bool_func *cond,
-                                  const Item *item) const override
+    Data_type_compatibility can_optimize_keypart_ref(const Item_bool_func *cond,
+                                                     const Item *item)
+                                                     const override
     {
       /*
         Mixing of two different non-traditional types is currently prevented.
@@ -633,23 +658,23 @@ public:
                   is_traditional_scalar_type() ||
                   item->type_handler()->type_collection() ==
                   type_handler()->type_collection());
-      return true;
+      return Data_type_compatibility::OK;
     }
     /**
       Test if Field can use range optimizer for a standard comparison operation:
         <=, <, =, <=>, >, >=
       Note, this method does not cover spatial operations.
     */
-    bool can_optimize_range(const Item_bool_func *cond,
-                            const Item *item,
-                            bool is_eq_func) const override
+    Data_type_compatibility can_optimize_range(const Item_bool_func *cond,
+                                               const Item *item,
+                                               bool is_eq_func) const override
     {
       // See the DBUG_ASSERT comment in can_optimize_keypart_ref()
       DBUG_ASSERT(item->type_handler()->type_handler_base_or_self()->
                   is_traditional_scalar_type() ||
                   item->type_handler()->type_collection() ==
                   type_handler()->type_collection());
-      return true;
+      return Data_type_compatibility::OK;
     }
     void hash_not_null(Hasher *hasher) override
     {
@@ -660,7 +685,8 @@ public:
                          scalar_comparison_op op, Item *value) override
     {
       DBUG_ENTER("Field_fbt::get_mm_leaf");
-      if (!can_optimize_scalar_range(prm, key_part, cond, op, value))
+      if (can_optimize_scalar_range(prm, key_part, cond, op, value) !=
+          Data_type_compatibility::OK)
         DBUG_RETURN(0);
       int err= value->save_in_field_no_warnings(this, 1);
       if ((op != SCALAR_CMP_EQUAL && is_real_null()) || err < 0)
@@ -673,15 +699,17 @@ public:
       }
       DBUG_RETURN(stored_field_make_mm_leaf(prm, key_part, op, value));
     }
-    bool can_optimize_hash_join(const Item_bool_func *cond,
-                                        const Item *item) const override
+    Data_type_compatibility can_optimize_hash_join(const Item_bool_func *cond,
+                                                   const Item *item)
+                                                   const override
     {
       return can_optimize_keypart_ref(cond, item);
     }
-    bool can_optimize_group_min_max(const Item_bool_func *cond,
+    Data_type_compatibility can_optimize_group_min_max(
+                                    const Item_bool_func *cond,
                                     const Item *const_item) const override
     {
-      return true;
+      return Data_type_compatibility::OK;
     }
 
     uint row_pack_length() const override { return pack_length(); }
@@ -744,9 +772,9 @@ public:
       Fbt_null tmp(arg);
       return m_null_value || tmp.is_null() ? UNKNOWN : m_native.cmp(tmp) != 0;
     }
-    int compare(cmp_item *ci) override
+    int compare(const cmp_item *ci) const override
     {
-      cmp_item_fbt *tmp= static_cast<cmp_item_fbt*>(ci);
+      const cmp_item_fbt *tmp= static_cast<const cmp_item_fbt*>(ci);
       DBUG_ASSERT(!m_null_value);
       DBUG_ASSERT(!tmp->m_null_value);
       return m_native.cmp(tmp->m_native);
@@ -760,13 +788,13 @@ public:
   class in_fbt :public in_vector
   {
     Fbt m_value;
-    static int cmp_fbt(void *cmp_arg, Fbt *a, Fbt *b)
+    static int cmp_fbt(void *cmp_arg, const void *a, const void *b)
     {
-      return a->cmp(*b);
+      return static_cast<const Fbt*>(a)->cmp(*static_cast<const Fbt*>(b));
     }
   public:
     in_fbt(THD *thd, uint elements)
-     :in_vector(thd, elements, sizeof(Fbt), (qsort2_cmp) cmp_fbt, 0),
+     :in_vector(thd, elements, sizeof(Fbt), cmp_fbt, 0),
       m_value(Fbt::zero())
     { }
     const Type_handler *type_handler() const override
@@ -778,10 +806,12 @@ public:
       Fbt *buff= &((Fbt *) base)[pos];
       Fbt_null value(item);
       if (value.is_null())
+      {
         *buff= Fbt::zero();
-      else
-        *buff= value;
-      return FALSE;
+        return true;
+      }
+      *buff= value;
+      return false;
     }
     uchar *get_value(Item *item) override
     {
@@ -848,8 +878,9 @@ public:
     {
       return Item::save_in_field(field, no_conversions);
     }
-    Item *get_copy(THD *thd) override
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_copy_fbt>(thd, this); }
+    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
   };
 
   class Item_char_typecast_func_handler_fbt_to_binary:
@@ -949,7 +980,7 @@ public:
       Fbt_null tmp(args[0]);
       return null_value= tmp.is_null() || tmp.to_native(to);
     }
-    Item *get_copy(THD *thd) override
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_typecast_fbt>(thd, this); }
   };
 
@@ -959,9 +990,10 @@ public:
   public:
     Item_cache_fbt(THD *thd)
      :Item_cache(thd, singleton()) { }
-    Item *get_copy(THD *thd)
+    Item *do_get_copy(THD *thd) const override
     { return get_item_copy<Item_cache_fbt>(thd, this); }
-    bool cache_value()
+    Item *do_build_clone(THD *thd) const override { return get_copy(thd); }
+    bool cache_value() override
     {
       if (!example)
         return false;
@@ -971,54 +1003,54 @@ public:
                                                    &m_value, type_handler());
       return true;
     }
-    String* val_str(String *to)
+    String* val_str(String *to) override
     {
       if (!has_value())
         return NULL;
       Fbt_null tmp(m_value.ptr(), m_value.length());
       return tmp.is_null() || tmp.to_string(to) ? NULL : to;
     }
-    my_decimal *val_decimal(my_decimal *to)
+    my_decimal *val_decimal(my_decimal *to) override
     {
       if (!has_value())
         return NULL;
       my_decimal_set_zero(to);
       return to;
     }
-    longlong val_int()
+    longlong val_int() override
     {
       if (!has_value())
         return 0;
       return 0;
     }
-    double val_real()
+    double val_real() override
     {
       if (!has_value())
         return 0;
       return 0;
     }
-    longlong val_datetime_packed(THD *thd)
+    longlong val_datetime_packed(THD *) override
     {
       DBUG_ASSERT(0);
       if (!has_value())
         return 0;
       return 0;
     }
-    longlong val_time_packed(THD *thd)
+    longlong val_time_packed(THD *) override
     {
       DBUG_ASSERT(0);
       if (!has_value())
         return 0;
       return 0;
     }
-    bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate)
+    bool get_date(THD *, MYSQL_TIME *ltime, date_mode_t) override
     {
       if (!has_value())
         return true;
       set_zero_time(ltime, MYSQL_TIMESTAMP_TIME);
       return false;
     }
-    bool val_native(THD *thd, Native *to)
+    bool val_native(THD *, Native *to) override
     {
       if (!has_value())
         return true;
@@ -1104,6 +1136,11 @@ public:
     return FbtImpl::max_char_length();
   }
 
+  const Type_handler *type_handler_for_implicit_upgrade() const override
+  {
+    return TypeCollectionImpl::singleton()->
+             type_handler_for_implicit_upgrade(this);
+  }
   const Type_handler *type_handler_for_comparison() const override
   {
     return this;
@@ -1528,6 +1565,16 @@ public:
     Fbt_null na(a), nb(b);
     return !na.is_null() && !nb.is_null() && !na.cmp(nb);
   }
+  bool Item_bool_rowready_func2_fix_length_and_dec(THD *thd,
+                                 Item_bool_rowready_func2 *func) const override
+  {
+    if (Type_handler::Item_bool_rowready_func2_fix_length_and_dec(thd, func))
+      return true;
+    if (!func->maybe_null() &&
+        Fbt::fix_fields_maybe_null_on_conversion_to_fbt(func->arguments(), 2))
+      func->set_maybe_null();
+    return false;
+  }
   bool Item_hybrid_func_fix_attributes(THD *thd, const LEX_CSTRING &name,
                                        Type_handler_hybrid_field_type *h,
                                        Type_all_attributes *attr,
@@ -1709,6 +1756,9 @@ public:
 
   bool Item_func_between_fix_length_and_dec(Item_func_between *func) const override
   {
+    if (!func->maybe_null() &&
+        Fbt::fix_fields_maybe_null_on_conversion_to_fbt(func->arguments(), 3))
+      func->set_maybe_null();
     return false;
   }
   longlong Item_func_between_val_int(Item_func_between *func) const override
@@ -1731,6 +1781,10 @@ public:
                                                     Item_func_in *func)
                                                     const override
   {
+    if (!func->maybe_null() &&
+        Fbt::fix_fields_maybe_null_on_conversion_to_fbt(func->arguments(),
+                                                        func->argument_count()))
+      func->set_maybe_null();
     if (func->compatible_types_scalar_bisection_possible())
     {
       return func->value_list_convert_const_to_int(thd) ||
@@ -1905,6 +1959,12 @@ public:
     return NULL;
   }
 
+  const Type_handler *type_handler_for_implicit_upgrade(
+                                               const Type_handler *from) const
+  {
+    return from;
+  }
+
   static Type_collection_fbt *singleton()
   {
     static Type_collection_fbt tc;
